'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var Highlight = require('prism-react-renderer');
var PropTypes = require('prop-types');
var React = require('react');
var cn = require('clsx');
var styled = require('styled-components');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () {
            return e[k];
          }
        });
      }
    });
  }
  n['default'] = e;
  return Object.freeze(n);
}

var Highlight__default = /*#__PURE__*/_interopDefaultLegacy(Highlight);
var PropTypes__default = /*#__PURE__*/_interopDefaultLegacy(PropTypes);
var React__default = /*#__PURE__*/_interopDefaultLegacy(React);
var cn__default = /*#__PURE__*/_interopDefaultLegacy(cn);
var styled__default = /*#__PURE__*/_interopDefaultLegacy(styled);

const Container = styled__default['default'].div `
  display: flex;
  align-items: stretch;
  overflow: hidden;
  width: 100%;
  padding: 0;
  transition: max-height 0.5s ease-in-out;
  tab-size: 2;
  -moz-osx-font-smoothing: grayscale;
  -webkit-font-smoothing: antialiased;
  font-smooth: always;
  white-space: pre;
  word-break: keep-all;
  word-spacing: normal;
  word-wrap: normal;

  .yozora-code-highlighter__line {
    display: block;
    min-width: fit-content;
    width: 100%;
    letter-spacing: inherit;
    line-height: inherit;
    overflow-wrap: inherit;
    tab-size: inherit;
    text-indent: inherit;
    text-rendering: inherit;
    text-transform: inherit;
    white-space: inherit;
    word-break: inherit;
    word-spacing: inherit;
    word-wrap: inherit;
    &.yozora-code-highlighter__line--highlight {
      background: var(--code-bg-highlight, hsla(210deg, 100%, 84%, 0.2));
      border-color: transparent;
    }
  }

  .yozora-code-highlighter__linenos {
    flex: 0 0 auto;
    overflow: hidden;
    padding: 0.5rem 0;
    cursor: default;
    user-select: none;
    text-align: right;
    border-right: 1px solid var(--code-color-border, hsla(0deg, 0%, 30%, 0.8));
    .yozora-code-highlighter__line {
      padding: 0 6px;
    }
  }
  .yozora-code-highlighter__codes {
    flex: 1 1 auto;
    overflow: auto;     // for css compatibility
    overflow: overlay;
    padding: 0.5rem 0;
    .yozora-code-highlighter__codes-wrapper {
      min-width: 100%;
      width: fit-content;
    }
    .yozora-code-highlighter__line {
      padding: 0 1rem 0 6px;
    }
  }
`;

const lineHeightRegex = /^([\d.]+)([\S]*)$/;
function calcHeight(lineHeight, countOfLines) {
    const value = Number(lineHeight);
    if (!Number.isNaN(value))
        return countOfLines * value;
    const m = lineHeightRegex.exec('' + lineHeight);
    if (m == null)
        return 0;
    const [, val, unit = 'px'] = m;
    return countOfLines * Number(val) + unit;
}

function HighlighterContent(props) {
    const { codesRef, highlightLinenos = [], collapsed = false, maxLines = -1, lineHeight = '1.6rem', showLinenos = true, tokens, getLineProps, getTokenProps, onLinenoWidthChange, className, style: _style, } = props;
    const linenoRef = React.useRef(null);
    const linenoWidth = showLinenos
        ? `${Math.max(2, ('' + tokens.length).length) * 1.1}em`
        : undefined;
    React.useEffect(() => {
        if (onLinenoWidthChange == null)
            return;
        onLinenoWidthChange(linenoWidth);
    }, [linenoWidth, onLinenoWidthChange]);
    const style = Object.assign(Object.assign({}, _style), { lineHeight, maxHeight: 0, backgroundColor: 'none' });
    if (!collapsed) {
        const countOfLines = maxLines > 0 ? maxLines : tokens.length;
        const maxHeight = calcHeight(lineHeight, countOfLines + 1);
        style.maxHeight = maxHeight;
        style.minHeight = '100%';
    }
    const syncScrollEvents = React.useCallback(e => {
        const codesArea = e.target;
        if (codesArea == null)
            return;
        const { scrollTop } = codesArea;
        if (linenoRef.current != null) {
            linenoRef.current.scrollTo(0, scrollTop);
        }
    }, []);
    return (React__default['default'].createElement(Container, { className: className, style: style },
        showLinenos && (React__default['default'].createElement("div", { key: "linenos", className: "yozora-code-highlighter__linenos", style: { width: linenoWidth }, ref: linenoRef }, tokens.map((_, lineNo) => {
            const isHighlight = highlightLinenos.includes(lineNo + 1);
            return (React__default['default'].createElement("div", { key: lineNo, className: cn__default['default']('yozora-code-highlighter__line', {
                    'yozora-code-highlighter__line--highlight': isHighlight,
                }) },
                React__default['default'].createElement("span", { key: lineNo }, lineNo + 1)));
        }))),
        React__default['default'].createElement("div", { key: "codes", ref: codesRef, className: "yozora-code-highlighter__codes", onScroll: syncScrollEvents },
            React__default['default'].createElement("div", { className: "yozora-code-highlighter__codes-wrapper" }, tokens.map((line, lineNo) => {
                const isHighlight = highlightLinenos.includes(lineNo + 1);
                const lineProps = getLineProps({ line });
                return (React__default['default'].createElement("div", Object.assign({}, lineProps, { key: lineNo, className: cn__default['default']('yozora-code-highlighter__line', {
                        'yozora-code-highlighter__line--highlight': isHighlight,
                    }, lineProps.className) }), line.map((token, key) => (React__default['default'].createElement("span", Object.assign({}, getTokenProps({ token }), { key: key }))))));
            })))));
}
HighlighterContent.displayName = 'YozoraCodeHighlighterContent';

const vscDarkTheme = {
    plain: {
        color: '#d4d4d4',
        backgroundColor: '#1e1e1e',
    },
    styles: [
        {
            types: ['prolog'],
            style: { color: 'rgb(0, 0, 128)' },
        },
        {
            types: ['comment', 'punctuation'],
            style: { color: 'rgb(106, 153, 85)' },
        },
        {
            types: ['builtin'],
            style: { color: 'rgb(79, 193, 255)' },
        },
        {
            types: ['number', 'variable', 'inserted'],
            style: { color: 'rgb(181, 206, 168)' },
        },
        {
            types: ['operator'],
            style: { color: 'rgb(212, 212, 212)' },
        },
        {
            types: ['constant'],
            style: { color: 'rgb(100, 102, 149)' },
        },
        {
            types: ['tag', 'changed', 'function', 'keyword'],
            style: { color: 'rgb(86, 156, 214)' },
        },
        {
            types: ['attr-name'],
            style: { color: 'rgb(156, 220, 254)' },
        },
        {
            types: ['deleted', 'string', 'attr-value'],
            style: { color: 'rgb(206, 145, 120)' },
        },
        {
            types: ['class-name'],
            style: { color: 'rgb(78, 201, 176)' },
        },
        {
            types: ['char'],
            style: { color: 'rgb(209, 105, 105)' },
        },
        {
            types: ['selector'],
            style: { color: 'rgb(215, 186, 125)' },
        },
        {
            types: ['tag'],
            languages: ['markup'],
            style: { color: 'rgb(86, 156, 214)' },
        },
        {
            types: ['punctuation'],
            languages: ['markup'],
            style: { color: '#808080' },
        },
    ],
};

const vscLightTheme = {
    plain: {
        color: '#000000',
        backgroundColor: '#ffffff',
    },
    styles: [
        {
            types: ['comment'],
            style: { color: 'rgb(0, 128, 0)' },
        },
        {
            types: ['builtin'],
            style: { color: 'rgb(0, 112, 193)' },
        },
        {
            types: ['number', 'variable', 'inserted'],
            style: { color: 'rgb(9, 134, 88)' },
        },
        {
            types: ['operator'],
            style: {
                color: 'rgb(0, 0, 0)',
            },
        },
        {
            types: ['constant', 'char'],
            style: { color: 'rgb(129, 31, 63)' },
        },
        {
            types: ['tag'],
            style: { color: 'rgb(128, 0, 0)' },
        },
        {
            types: ['attr-name'],
            style: { color: 'rgb(255, 0, 0)' },
        },
        {
            types: ['deleted', 'string'],
            style: { color: 'rgb(163, 21, 21)' },
        },
        {
            types: ['changed', 'punctuation'],
            style: { color: 'rgb(4, 81, 165)' },
        },
        {
            types: ['function', 'keyword'],
            style: { color: 'rgb(0, 0, 255)' },
        },
        {
            types: ['class-name'],
            style: { color: 'rgb(38, 127, 153)' },
        },
    ],
};

function CodeHighlighter(props) {
    const { lang, value: code, darken = true, highlightLinenos, maxLines, lineHeight, collapsed, showLinenos, codesRef, onLinenoWidthChange, } = props;
    const theme = props.theme == null ? (darken ? vscDarkTheme : vscLightTheme) : props.theme;
    return (React__default['default'].createElement(Highlight__default['default'], { Prism: Highlight.Prism, code: code, theme: theme, language: lang }, ({ tokens, getLineProps, getTokenProps, className, style }) => (React__default['default'].createElement(HighlighterContent, { codesRef: codesRef, highlightLinenos: highlightLinenos, lineHeight: lineHeight, maxLines: maxLines, collapsed: collapsed, showLinenos: showLinenos, tokens: tokens, getLineProps: getLineProps, getTokenProps: getTokenProps, onLinenoWidthChange: onLinenoWidthChange, className: className, style: style }))));
}
CodeHighlighter.displayName = 'YozoraCodeHighlighter';
CodeHighlighter.propTypes = {
    codesRef: PropTypes__default['default'].any,
    collapsed: PropTypes__default['default'].bool,
    darken: PropTypes__default['default'].bool,
    highlightLinenos: PropTypes__default['default'].arrayOf(PropTypes__default['default'].number),
    lang: PropTypes__default['default'].string,
    lineHeight: PropTypes__default['default'].oneOfType([PropTypes__default['default'].string, PropTypes__default['default'].number]),
    maxLines: PropTypes__default['default'].number,
    onLineWidthChange: PropTypes__default['default'].func,
    showLinenos: PropTypes__default['default'].bool,
    theme: PropTypes__default['default'].object,
    value: PropTypes__default['default'].string.isRequired,
};

function enablePrismLanguages() {
    const self = typeof global !== 'undefined' ? global : window;
    self.Prism = Highlight.Prism;
    Promise.resolve().then(function () { return /*#__PURE__*/_interopNamespace(require('prismjs/components/prism-visual-basic')); });
}
enablePrismLanguages();

exports.CodeHighlighter = CodeHighlighter;
exports.HighlighterContent = HighlighterContent;
exports.default = CodeHighlighter;
exports.vscDarkTheme = vscDarkTheme;
exports.vscLightTheme = vscLightTheme;
